@@ -1,6 +1,7 @@
 /*
  * libwbxml, the WBXML Library.
- * Copyright (C) 2002-2005 Aymerick Jehanne <aymerick@jehanne.org>
+ * Copyright (C) 2002-2008 Aymerick Jehanne <aymerick@jehanne.org>
+ * Copyright (C) 2011 Michael Bell <michael.bell@opensync.org>
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,7 +19,7 @@
  * 
  * LGPL v2.1: http://www.gnu.org/copyleft/lesser.txt
  * 
- * Contact: libwbxml@aymerick.com
+ * Contact: aymerick@jehanne.org
  * Home: http://libwbxml.aymerick.com
  */
  
@@ -26,13 +27,14 @@
  * @file wbxml_errors.c
  * @ingroup wbxml_errors
  *
- * @author Aymerick Jehanne <libwbxml@aymerick.com>
+ * @author Aymerick Jehanne <aymerick@jehanne.org>
  * @date 02/11/18
  *
  * @brief WBXML Error Codes Handling
  */
 
-#include "wbxml.h"
+#include "wbxml_errors.h"
+#include <stdio.h>
 
 
 /**
@@ -52,7 +54,7 @@ static const WBXMLErrorCodeItem error_table [] = {
     { WBXML_OK,                                 "No Error" },
     { WBXML_NOT_ENCODED,                        "Not Encoded" },
     { WBXML_ERROR_ATTR_TABLE_UNDEFINED,         "Attribute Table Undefined" },
-    { WBXML_ERROR_BAD_DATETIME,                 "Bad %Datetime Format" },
+    { WBXML_ERROR_BAD_DATETIME,                 "Bad Datetime Format" },
     { WBXML_ERROR_BAD_PARAMETER,                "Bad Parameter" },
     { WBXML_ERROR_INTERNAL,                     "Internal Error" },
     { WBXML_ERROR_LANG_TABLE_UNDEFINED,         "Languages Table Undefined" },
@@ -60,6 +62,7 @@ static const WBXMLErrorCodeItem error_table [] = {
     { WBXML_ERROR_NOT_IMPLEMENTED,              "Not Implemented" },
     { WBXML_ERROR_TAG_TABLE_UNDEFINED,          "Tag Table Undefined" },
     { WBXML_ERROR_B64_ENC,                      "Failed to encode Base64" },
+    { WBXML_ERROR_B64_DEC,                      "Failed to decode Base64" },
 #if defined( WBXML_SUPPORT_WV )
     { WBXML_ERROR_WV_DATETIME_FORMAT,           "Bad Wireless-Village Date and Time Format" },
 #endif /* WBXML_SUPPORT_WV */
@@ -94,16 +97,21 @@ static const WBXMLErrorCodeItem error_table [] = {
     /* WBXML Encoder Errors */
     { WBXML_ERROR_ENCODER_APPEND_DATA,          "Can't append data to output buffer" },
     { WBXML_ERROR_STRTBL_DISABLED,              "String Table generation disabled: can't encode Literal" },
+    { WBXML_ERROR_UNKNOWN_XML_LANGUAGE,		"The XML language is unknown." },
     { WBXML_ERROR_XML_NODE_NOT_ALLOWED,         "XML Node Type not allowed" },
     { WBXML_ERROR_XML_NULL_ATTR_NAME,           "NULL XML Attribute Name" },
     { WBXML_ERROR_XML_PARSING_FAILED,           "Parsing of XML Document Failed" },
 #if defined( WBXML_SUPPORT_SYNCML )
-    { WBXML_ERROR_XML_DEVINF_CONV_FAILED,       "Convertion of XML DevInf document Failed" },
+    { WBXML_ERROR_XML_DEVINF_CONV_FAILED,       "The conversion of a XML DevInf document failed" },
 #endif /* WBXML_SUPPORT_WV */
     { WBXML_ERROR_NO_XMLPARSER,                 "Can't parse XML (Compiled without Expat): XML to WBXML conversion not possible" },
     { WBXML_ERROR_XMLPARSER_OUTPUT_UTF16,       "XML Parser (Expat) is outputting UTF-16, so you have two choices :\n"
                                               "  - Get the iconv library (http://www.gnu.org/software/libiconv/)\n"
-                                              "  - Use a version of Expat that output UTF-8" }
+                                              "  - Use a version of Expat that output UTF-8" },
+    { WBXML_ERROR_CHARSET_UNKNOWN,              "The character set is unknown."},
+    { WBXML_ERROR_CHARSET_CONV_INIT,            "The converter for the character set cannot be initialized."},
+    { WBXML_ERROR_CHARSET_CONV,                 "The character conversion failed."},
+    { WBXML_ERROR_CHARSET_NOT_FOUND,            "The character set cannot be found."}
 };
 
 #define ERROR_TABLE_SIZE ((WB_ULONG) (sizeof(error_table) / sizeof(error_table[0])))
@@ -122,5 +130,7 @@ WBXML_DECLARE(const WB_UTINY *) wbxml_errors_string(WBXMLError error_code)
             return (const WB_UTINY *)error_table[i].string;
     }
 
+    fprintf(stderr, "PACKAGE_NAME: Unknown error code %d.\n", error_code);
+
     return (const WB_UTINY *)"Unknown Error Code";
 }
