@@ -1,6 +1,7 @@
 /*
  * libwbxml, the WBXML Library.
- * Copyright (C) 2002-2005 Aymerick Jehanne <aymerick@jehanne.org>
+ * Copyright (C) 2002-2008 Aymerick Jehanne <aymerick@jehanne.org>
+ * Copyright (C) 2009-2011 Michael Bell <michael.bell@opensync.org>
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,7 +19,7 @@
  * 
  * LGPL v2.1: http://www.gnu.org/copyleft/lesser.txt
  * 
- * Contact: libwbxml@aymerick.com
+ * Contact: aymerick@jehanne.org
  * Home: http://libwbxml.aymerick.com
  */
  
@@ -26,7 +27,7 @@
  * @file wbxml_parser.c
  * @ingroup wbxml_parser
  *
- * @author Aymerick Jehanne <libwbxml@aymerick.com>
+ * @author Aymerick Jehanne <aymerick@jehanne.org>
  * @date 02/03/12
  *
  * @brief WBXML Parser - Parse a WBXML document and call user defined Callbacks
@@ -42,7 +43,12 @@
  *                         - A strange 'body' rule description in BNF (just forget it).
  */
 
-#include "wbxml.h"
+#include "wbxml_parser.h"
+#include "wbxml_log.h"
+#include "wbxml_internals.h"
+#include "wbxml_base64.h"
+#include "wbxml_charset.h"
+#include <assert.h>
 
 
 /* Memory management related defines */
@@ -56,10 +62,6 @@
 /** For unknown Tag Name or Attribute Name (in Best Effort Mode) */
 #define WBXML_PARSER_UNKNOWN_STRING ((WB_UTINY *)"unknown")
 
-/** If you want to modify this define, change the 'entcode' variable length in parse_entity() too please */
-#define WBXML_PARSER_MAX_ENTITY_CODE 999999
-
-
 /**
  * @brief The WBXML Application Token types
  */
@@ -153,11 +155,14 @@ static WBXMLError parse_uint8(WBXMLParser *parser, WB_UTINY *result);
 static WBXMLError parse_mb_uint32(WBXMLParser *parser, WB_ULONG *result);
 
 /* Language Specific Decoding Functions */
+static WBXMLError decode_base64_value(WBXMLBuffer **data);
+
 #if defined( WBXML_SUPPORT_SI ) || defined( WBXML_SUPPORT_EMN )
 static WBXMLError decode_datetime(WBXMLBuffer *buff);
 #endif /* WBXML_SUPPORT_SI || WBXML_SUPPORT_EMN */
 
 static WBXMLError decode_opaque_content(WBXMLParser *parser, WBXMLBuffer **data);
+static WBXMLError decode_opaque_attr_value(WBXMLParser *parser, WBXMLBuffer **data);
 
 #if defined( WBXML_SUPPORT_WV )
 static WBXMLError decode_wv_content(WBXMLParser *parser, WBXMLBuffer **data);
@@ -165,10 +170,6 @@ static WBXMLError decode_wv_integer(WBXMLBuffer **data);
 static WBXMLError decode_wv_datetime(WBXMLBuffer **data);
 #endif /* WBXML_SUPPORT_WV */
 
-#if defined( WBXML_SUPPORT_DRMREL )
-static WBXMLError decode_drmrel_keyvalue(WBXMLBuffer **data);
-#endif /* WBXML_SUPPORT_DRMREL */
-
 /* Macro for error handling */
 #define CHECK_ERROR if (ret != WBXML_OK) return ret;
 
@@ -181,7 +182,7 @@ WBXML_DECLARE(WBXMLParser *) wbxml_parser_create(void)
 {
     WBXMLParser *parser = NULL;
 
-    parser = (WBXMLParser *) wbxml_malloc(sizeof(WBXMLParser));
+    parser = wbxml_malloc(sizeof(WBXMLParser));
     if (parser == NULL) {
         return NULL;
     }
@@ -245,8 +246,9 @@ WBXML_DECLARE(WBXMLError) wbxml_parser_parse(WBXMLParser *parser, WB_UTINY *wbxm
     ret = parse_version(parser);
     CHECK_ERROR
 
-    if ((WB_UTINY)parser->version > WBXML_VERSION_13)
-        WBXML_WARNING((WBXML_PARSER, "This library only supports WBXML " WBXML_VERSION_TEXT_13));
+    if ((WB_UTINY)parser->version > WBXML_VERSION_13) {
+        WBXML_WARNING((WBXML_PARSER, "This library only supports WBXML %s.", WBXML_VERSION_TEXT_13));
+    }
 
     /* WBXML Public ID */
     ret = parse_publicid(parser);
@@ -277,7 +279,7 @@ WBXML_DECLARE(WBXMLError) wbxml_parser_parse(WBXMLParser *parser, WB_UTINY *wbxm
             parser->charset = WBXML_PARSER_DEFAULT_CHARSET;
       
             WBXML_WARNING((WBXML_PARSER,
-                           "No charset information found, using default : %s",
+                           "No charset information found, using default : %x",
                            WBXML_PARSER_DEFAULT_CHARSET));
         }
     }
@@ -419,7 +421,6 @@ static void wbxml_parser_reinit(WBXMLParser *parser)
     parser->public_id       = WBXML_PUBLIC_ID_UNKNOWN;    
     parser->public_id_index = -1;
     parser->charset         = WBXML_CHARSET_UNKNOWN;
-    parser->meta_charset    = WBXML_CHARSET_UNKNOWN;
     parser->version         = WBXML_VERSION_UNKNOWN;    
   
     parser->pos             = 0;
@@ -546,6 +547,7 @@ static WB_BOOL check_public_id(WBXMLParser *parser)
 {
     WBXMLBuffer *public_id = NULL;
     WB_LONG          index = 0;
+    WBXMLError         ret;
 
     WBXML_DEBUG((WBXML_PARSER, "\t  Checking PublicID"));
 
@@ -564,7 +566,7 @@ static WB_BOOL check_public_id(WBXMLParser *parser)
 
     if (parser->lang_forced != WBXML_LANG_UNKNOWN) {
         /* Search Language Entry */
-        while (parser->mainTable[index].langID != -1) {
+        while (parser->mainTable[index].langID != WBXML_LANG_UNKNOWN) {
             if (parser->mainTable[index].langID == parser->lang_forced) {
                 parser->langTable = &(parser->mainTable[index]);
 
@@ -607,8 +609,9 @@ static WB_BOOL check_public_id(WBXMLParser *parser)
     if (parser->public_id_index != -1) {
         WBXML_DEBUG((WBXML_PARSER, "\t  PublicID is in String Table (index: 0x%X)", parser->public_id_index));
 
-        if (get_strtbl_reference(parser, (WB_ULONG) parser->public_id_index, &public_id) != WBXML_OK) {
-            WBXML_ERROR((WBXML_PARSER, "Bad publicID reference in string table"));
+        ret = get_strtbl_reference(parser, (WB_ULONG) parser->public_id_index, &public_id);
+        if (ret != WBXML_OK) {
+            WBXML_ERROR((WBXML_PARSER, "Bad publicID reference in string table. %s", wbxml_errors_string(ret)));
             return FALSE;
         }
 
@@ -659,12 +662,14 @@ static WBXMLError parse_version(WBXMLParser *parser)
      * Do NOT keep 'WBXML_VERSION_UNKNOWN' (0xffffffff) because only one byte will change.
      * (for example, if the version is 0x02, then parser->version will be 0xffffff02)
      */
-    parser->version = WBXML_VERSION_10;
+    WB_UTINY version = WBXML_VERSION_10;
     
-    if ((ret = parse_uint8(parser, (WB_UTINY*) &parser->version)) != WBXML_OK)
+    if ((ret = parse_uint8(parser, &version)) != WBXML_OK)
         return ret;
+    
+    parser->version = version;
 
-    WBXML_DEBUG((WBXML_PARSER, "(%d) Parsed version: '0x%X'", parser->pos - 1, (WB_TINY) parser->version));
+    WBXML_DEBUG((WBXML_PARSER, "(%d) Parsed version: 1.%d", parser->pos - 1, parser->version));
 
     return WBXML_OK;
 }
@@ -680,21 +685,26 @@ static WBXMLError parse_version(WBXMLParser *parser)
 static WBXMLError parse_publicid(WBXMLParser *parser)
 {
     WB_UTINY public_id;
+    WBXMLError ret;
 
     if (!wbxml_buffer_get_char(parser->wbxml, parser->pos, &public_id))
         return WBXML_ERROR_END_OF_BUFFER;
 
-    WBXML_DEBUG((WBXML_PARSER, "(%d) Parsed publicid: '0x%X'", parser->pos, public_id));
+    WBXML_DEBUG((WBXML_PARSER, "(%d) Parsed public id value: '0x%X'", parser->pos, public_id));
 
     if (public_id == 0x00) {
         parser->pos++;
 
-        /* Get index (we will retreive the Public ID latter) */
-        return parse_mb_uint32(parser, (WB_ULONG *)&parser->public_id_index);
+        /* Get index (we will retrieve the Public ID later from string table) */
+        ret = parse_mb_uint32(parser, (WB_ULONG *)&parser->public_id_index);
+        WBXML_DEBUG((WBXML_PARSER, "(%d) Parsed public id index: '0x%x'", parser->pos-1, parser->public_id_index));
+        return ret;
     }
     else {
         /* Get Public ID */
-        return parse_mb_uint32(parser, &parser->public_id);        
+        ret = parse_mb_uint32(parser, &parser->public_id);        
+        WBXML_DEBUG((WBXML_PARSER, "(%d) Parsed public id: '0x%x'", parser->pos-1, parser->public_id));
+        return ret;
     }
 }
 
@@ -715,15 +725,43 @@ static WBXMLError parse_publicid(WBXMLParser *parser)
  */
 static WBXMLError parse_charset(WBXMLParser *parser)
 {
+    /* definitions first ... or some compilers don't like it */
+    const char *charset_name = NULL;
+
 #if defined( WBXML_LIB_VERBOSE )
     WB_ULONG startpos = parser->pos;
 #endif /* WBXML_LIB_VERBOSE */
 
-    WBXMLError ret = parse_mb_uint32(parser, (WB_ULONG *)&parser->charset);
+    unsigned int charset = 0;
+    WBXMLError ret = parse_mb_uint32(parser, &charset);
+
+    if (ret != WBXML_OK) {
+        WBXML_DEBUG((WBXML_PARSER, "(%d) failed to parse character set", startpos));
+        return ret;
+    }
+
+    if (charset == 0) {
+        WBXML_DEBUG((WBXML_PARSER, "(%d) The character set is zero.", startpos));
+        if (parser->meta_charset != WBXML_CHARSET_UNKNOWN) {
+            /* use character set from transport meta-information */
+            WBXML_DEBUG((WBXML_PARSER, "(%d) Using charset from meta-info ...%d.", startpos, parser->meta_charset));
+            charset = parser->meta_charset;
+        } else {
+            /* default encoding is UTF-8 */
+            WBXML_DEBUG((WBXML_PARSER, "(%d) Enabling the default character set ... UTF-8.", startpos));
+            charset = WBXML_CHARSET_UTF_8;
+        }
+    }
+
+    if (!wbxml_charset_get_name(charset, &charset_name)) {
+        WBXML_DEBUG((WBXML_PARSER, "(%d) failed to get character set name", charset));
+        return WBXML_ERROR_CHARSET_NOT_FOUND;
+    }
+    parser->charset = (WBXMLCharsetMIBEnum) charset;
 
     WBXML_DEBUG((WBXML_PARSER, "(%d) Parsed charset: '0x%X'", startpos, parser->charset));
 
-    return ret;
+    return WBXML_OK;
 }
 
 
@@ -750,7 +788,7 @@ static WBXMLError parse_strtbl(WBXMLParser *parser)
 
     if (strtbl_len > 0) {
         /* Check this string table length */
-        if (parser->pos + strtbl_len > wbxml_buffer_len(parser->wbxml))
+        if (strtbl_len > wbxml_buffer_len(parser->wbxml) - parser->pos)
             return WBXML_ERROR_STRTBL_LENGTH;
 
         /* Get String Table */
@@ -769,8 +807,27 @@ static WBXMLError parse_strtbl(WBXMLParser *parser)
             return WBXML_ERROR_INTERNAL;
         }
 
+        /* If there is a correctly terminated string
+         * then there is no reason to do anything.
+         */
         if (end_char != '\0') {
-            /* Append NULL char to end of String Table */
+            WBXML_DEBUG((WBXML_PARSER, "    Adding NULL bytes to strtbl."));
+            /* A terminating NULL byte is not enough today.
+             * There exists character sets with a fixed length of bytes.
+             * Such strings must be terminated with a complete NULL character.
+             * ASCII  => 1 byte
+             * UTF-8  => 1 byte
+             * UTF-16 => 2 bytes
+             * UTF-32 => 4 bytes (I hope so)
+             * UCS-2  => 2 bytes
+             * I hope four NULL bytes are enough for all cases.
+             */
+            if (!wbxml_buffer_append_char(parser->strstbl, '\0'))
+                return WBXML_ERROR_NOT_ENOUGH_MEMORY;
+            if (!wbxml_buffer_append_char(parser->strstbl, '\0'))
+                return WBXML_ERROR_NOT_ENOUGH_MEMORY;
+            if (!wbxml_buffer_append_char(parser->strstbl, '\0'))
+                return WBXML_ERROR_NOT_ENOUGH_MEMORY;
             if (!wbxml_buffer_append_char(parser->strstbl, '\0'))
                 return WBXML_ERROR_NOT_ENOUGH_MEMORY;
         }
@@ -938,6 +995,8 @@ static WBXMLError parse_element(WBXMLParser *parser)
     if ((ret = parse_stag(parser, &tag, &element)) != WBXML_OK ) {
         return ret;
     }
+
+    WBXML_DEBUG((WBXML_PARSER, "<%s>", wbxml_tag_get_xml_name(element)));
   
     /* Set Current Tag */
     if (element->type == WBXML_VALUE_TOKEN) {
@@ -987,14 +1046,13 @@ static WBXMLError parse_element(WBXMLParser *parser)
     {
         parser->content_hdl->start_element_clb(parser->user_data,
                                                element,
-                                               attrs,
-                                               is_empty);
+                                               attrs);
     }
-      
+    
     /* Free Attributes */
     free_attrs_table(attrs);
     
-      
+    
     /* Parse *content */
     if (!is_empty) {
         /* There can be NO content */
@@ -1023,7 +1081,7 @@ static WBXMLError parse_element(WBXMLParser *parser)
             content = NULL;
         }
         
-        WBXML_DEBUG((WBXML_PARSER, "(%d) End of Element", parser->pos));
+        WBXML_DEBUG((WBXML_PARSER, "(%d) End of Element", parser->pos - 1));
         
         /* Skip END */
         parser->pos++;
@@ -1034,10 +1092,11 @@ static WBXMLError parse_element(WBXMLParser *parser)
         (parser->content_hdl->end_element_clb != NULL))
     {
         parser->content_hdl->end_element_clb(parser->user_data,
-                                             element,
-                                             is_empty);
+                                             element);
     }
-      
+    
+    WBXML_DEBUG((WBXML_PARSER, "</%s>", wbxml_tag_get_xml_name(element)));
+    
     /* Free Tag */
     wbxml_tag_destroy(element);
       
@@ -1077,8 +1136,9 @@ static WBXMLError parse_switch_page(WBXMLParser *parser, WBXMLTokenType code_spa
 {
     WBXML_DEBUG((WBXML_PARSER, "(%d) Parsing switchPage", parser->pos));
 
-    if ((WB_UTINY) parser->version < (WB_UTINY) WBXML_VERSION_12)
-        WBXML_WARNING((WBXML_PARSER, "No Switch Page mecanism possible in WBXML < %s", WBXML_VERSION_TEXT_12));
+    if ((WB_UTINY) parser->version < (WB_UTINY) WBXML_VERSION_12) {
+        WBXML_WARNING((WBXML_PARSER, "No Switch Page mecanism possible in WBXML < %s (current: %d)", WBXML_VERSION_TEXT_12, (WB_UTINY) parser->version));
+    }
 
     /* Skip SWITCH_PAGE token */
     parser->pos++;
@@ -1456,7 +1516,9 @@ static WBXMLError parse_extension(WBXMLParser *parser, WBXMLTokenType code_space
         return ret;
     }
 
-
+    if (parser->langTable == NULL) {
+            return WBXML_ERROR_LANG_TABLE_UNDEFINED;
+        }
     /* Language specific treatment */
     switch (parser->langTable->langID) {
 
@@ -1520,11 +1582,12 @@ static WBXMLError parse_extension(WBXMLParser *parser, WBXMLTokenType code_space
         }
 
         /* Build Variable */
-        ext = (WB_UTINY*) wbxml_malloc(WBXML_STRLEN(var_begin) +
+        ext = wbxml_malloc(WBXML_STRLEN(var_begin) +
                                        wbxml_buffer_len(var_value) +
                                        WBXML_STRLEN(escape) +
                                        WBXML_STRLEN(var_end) + 1);
         if (ext == NULL) {
+            wbxml_buffer_destroy(var_value);
             return WBXML_ERROR_NOT_ENOUGH_MEMORY;
         }
     
@@ -1562,6 +1625,7 @@ static WBXMLError parse_extension(WBXMLParser *parser, WBXMLTokenType code_space
             break;
 
         default:
+            wbxml_free(ext);
             return WBXML_ERROR_UNKNOWN_EXTENSION_TOKEN;
         }
     
@@ -1593,10 +1657,6 @@ static WBXMLError parse_extension(WBXMLParser *parser, WBXMLTokenType code_space
         }
 
         /* Search Token in Extension Value Table */
-        if (parser->langTable == NULL) {
-            return WBXML_ERROR_LANG_TABLE_UNDEFINED;
-        }
-    
         if (parser->langTable->extValueTable == NULL) {
             return WBXML_ERROR_EXT_VALUE_TABLE_UNDEFINED;
         }
@@ -1613,6 +1673,7 @@ static WBXMLError parse_extension(WBXMLParser *parser, WBXMLTokenType code_space
 #if WBXML_PARSER_BEST_EFFORT
             ext = (WB_UTINY *) wbxml_strdup((const WB_TINY*) WBXML_PARSER_UNKNOWN_STRING);
             len = WBXML_STRLEN(WBXML_PARSER_UNKNOWN_STRING);
+            wbxml_free(ext);
             return WBXML_OK;
 #else
             return WBXML_ERROR_UNKNOWN_EXTENSION_VALUE;
@@ -1635,6 +1696,7 @@ static WBXMLError parse_extension(WBXMLParser *parser, WBXMLTokenType code_space
     }
     else {
         if ((*result = wbxml_buffer_create(ext, len, len)) == NULL) {
+            wbxml_free(ext);
             return WBXML_ERROR_NOT_ENOUGH_MEMORY;
         }
 
@@ -1661,7 +1723,6 @@ static WBXMLError parse_extension(WBXMLParser *parser, WBXMLTokenType code_space
  */
 static WBXMLError parse_entity(WBXMLParser *parser, WBXMLBuffer **result)
 {
-    WB_TINY    entity[10];
     WB_ULONG   code   = 0;
     WBXMLError ret    = WBXML_OK;
   
@@ -1674,23 +1735,44 @@ static WBXMLError parse_entity(WBXMLParser *parser, WBXMLBuffer **result)
         return ret;
     }
   
-    /* Build Entity */
-    if ( code > WBXML_PARSER_MAX_ENTITY_CODE )  {
-        return WBXML_ERROR_ENTITY_CODE_OVERFLOW;
-    }
-  
-    /**
-     * WARNING: If you change the entity variable length (10 chars), change too
-     *          'WBXML_PARSER_MAX_ENTITY_CODE' defined in this file !
+    /*
+     * Convert the UCS-4 code to a UTF-8 encoded string.
      */
-    sprintf(entity, "&#%u;", code);
 
-    /* Create result buffer */
-    if ( (*result = wbxml_buffer_create_from_cstr(entity)) == NULL ) {
-        return WBXML_ERROR_NOT_ENOUGH_MEMORY;
+    assert(code < 0x80000000);
+
+    if (code < 0x80)
+    {
+        /* For codes under 0x80, we don't need any fancy formatting. */
+        WB_TINY entity[2] = {(WB_TINY)code, 0};
+
+        /* Create result buffer */
+        if ( (*result = wbxml_buffer_create_from_cstr(entity)) == NULL ) {
+            return WBXML_ERROR_NOT_ENOUGH_MEMORY;
+        }
+      
+        return WBXML_OK;
+    }
+    else
+    {
+        WB_UTINY masks[5] = {0xFC, 0xF8, 0xF0, 0xE0, 0xC0};
+        WB_UTINY entity[7] = {0, 0, 0, 0, 0, 0, 0};
+
+        int index = 5;
+        while (code >= 0x40)
+        {
+            entity[index] = 0x80 | (code & 0x3F);
+            code >>= 6; index--;
+        }
+        entity[index] = masks[index] | code;
+
+        /* Create result buffer */
+        if ( (*result = wbxml_buffer_create_from_cstr((WB_TINY *) entity + index)) == NULL ) {
+            return WBXML_ERROR_NOT_ENOUGH_MEMORY;
+        }
+      
+        return WBXML_OK;        
     }
-  
-  return WBXML_OK;
 }
 
 
@@ -1709,8 +1791,9 @@ static WBXMLError parse_opaque(WBXMLParser *parser, WBXMLBuffer **result)
   
     WBXML_DEBUG((WBXML_PARSER, "(%d) Parsing opaque", parser->pos));
   
-    if (parser->version < WBXML_VERSION_11)
+    if (parser->version < WBXML_VERSION_11) {
         WBXML_WARNING((WBXML_PARSER, "No 'opaque' support in WBXML < %s", WBXML_VERSION_TEXT_11));
+    }
   
     /* Skip OPAQUE */
     parser->pos++;
@@ -1720,7 +1803,7 @@ static WBXMLError parse_opaque(WBXMLParser *parser, WBXMLBuffer **result)
     }
 
     /* Check that length specified in OPAQUE doesn't overflow wbxml length */
-    if (parser->pos + len > wbxml_buffer_len(parser->wbxml)) {
+    if (len > wbxml_buffer_len(parser->wbxml) - parser->pos) {
         return WBXML_ERROR_BAD_OPAQUE_LENGTH;
     }
 
@@ -1946,7 +2029,10 @@ static WBXMLError parse_attr_value(WBXMLParser  *parser,
             WBXML_ERROR((WBXML_PARSER, "An Attribute value can't be 'opaque' in WBXML version < %s", WBXML_VERSION_TEXT_12));
         }
         
-        return parse_opaque(parser, result);
+        if ((ret = parse_opaque(parser, result)) != WBXML_OK) 
+            return ret;
+        
+        return decode_opaque_attr_value(parser, result);
     }
   
   
@@ -2105,6 +2191,32 @@ static WBXMLError get_strtbl_reference(WBXMLParser  *parser,
 {
     WB_ULONG   max_len = 0;
     WBXMLError ret     = WBXML_OK;
+
+    /* WORKAROUND: 2011-Jan-21 Michael Bell
+     * WORKAROUND:
+     * WORKAROUND: Nokia encodes the name space not only in the publicid.
+     * WORKAROUND: Some Nokia software encodes the SyncML name space
+     * WORKAROUND: as an attribute without a name and without a string
+     * WORKAROUND: table but with a LITAERAL and an index.
+     * WORKAROUND:
+     * WORKAROUND: Example:  ED 04 00 03 SYNCML:SYNCML1.2 00
+     * WORKAROUND:   ED => 2D => SyncML
+     * WORKAROUND:   04 => LITERAL (incl. ATTRSTART)
+     * WORKAROUND:   00 => index 0 (without string table)
+     * WORKAROUND:   03 => TOKEN_STR_I (inline sring)
+     * WORKAROUND:
+     * WORKAROUND: If this mistake is detected then "xmlns" is returned.
+     */
+    if (parser->strstbl == NULL && index == 0) {
+        WBXML_DEBUG((WBXML_PARSER, "(%d) Workaround Nokia: NO string table, index 0 => encoded xmlns", parser->pos));
+
+        /* UTF-8 xmlns */
+        if ((*result = wbxml_buffer_create_from_cstr("xmlns")) == NULL) {
+            return WBXML_ERROR_NOT_ENOUGH_MEMORY;
+        }
+        
+        return WBXML_OK;
+    }
   
     /* Check if strtbl is NULL */
     if (parser->strstbl == NULL) {
@@ -2117,7 +2229,7 @@ static WBXMLError get_strtbl_reference(WBXMLParser  *parser,
 
     /* Get max possible string length */
     max_len = wbxml_buffer_len(parser->strstbl) - index;
-  
+
     /* Convert to UTF-8 Buffer */
     if ((ret = wbxml_charset_conv_term((const WB_TINY *) (wbxml_buffer_get_cstr(parser->strstbl) + index),
                                        &max_len,
@@ -2126,9 +2238,9 @@ static WBXMLError get_strtbl_reference(WBXMLParser  *parser,
                                        WBXML_CHARSET_UTF_8)) != WBXML_OK) {
         return ret;
     }
-  
+
     WBXML_DEBUG((WBXML_PARSER, "(%d) String Table Reference: %s", parser->pos, wbxml_buffer_get_cstr(*result)));
-  
+
     return WBXML_OK;
 }
 
@@ -2209,6 +2321,40 @@ static WBXMLError parse_mb_uint32(WBXMLParser *parser, WB_ULONG *result)
  * Language Specific Decoding Functions 
  */
 
+/**
+ * @brief Decode a BASE64 value
+ * @param data [in/out]The value to decode
+ * @return WBXML_OK if OK, another error code otherwise
+ */
+static WBXMLError decode_base64_value(WBXMLBuffer **data)
+{
+    WB_UTINY   *result = NULL;
+    WBXMLError  ret    = WBXML_OK;
+    
+    if ((data == NULL) || (*data == NULL)) {
+        return WBXML_ERROR_INTERNAL;
+    }
+    
+    if ((result = wbxml_base64_encode((const WB_UTINY *) wbxml_buffer_get_cstr(*data),
+                                      wbxml_buffer_len(*data))) == NULL)
+    {
+        return WBXML_ERROR_B64_ENC;
+    }
+    
+    /* Reset buffer */
+    wbxml_buffer_delete(*data, 0, wbxml_buffer_len(*data));
+    
+    /* Set data */
+    if (!wbxml_buffer_append_cstr(*data, result)) {
+        ret = WBXML_ERROR_NOT_ENOUGH_MEMORY;
+    }
+    
+    wbxml_free(result);
+    
+    return ret;
+}
+
+
 #if ( defined( WBXML_SUPPORT_SI ) || defined( WBXML_SUPPORT_EMN ) )
 
 /**************************************
@@ -2297,8 +2443,6 @@ static WBXMLError decode_datetime(WBXMLBuffer *buff)
  * @param parser The WBXML Parser
  * @param data The Opaque data buffer
  * @return WBXML_OK if OK, another error code otherwise
- * @note Used for:
- *      - WV 1.1 / 1.2
  */
 static WBXMLError decode_opaque_content(WBXMLParser  *parser,
                                         WBXMLBuffer **data)
@@ -2318,21 +2462,67 @@ static WBXMLError decode_opaque_content(WBXMLParser  *parser,
 #if defined( WBXML_SUPPORT_DRMREL )
 
     case WBXML_LANG_DRMREL10:
+        /* ds:KeyValue */
         if ((parser->current_tag->wbxmlCodePage == 0x00) &&
             (parser->current_tag->wbxmlToken == 0x0C))
         {
-            WBXMLError ret = WBXML_OK;
-            
-            /* Decode <KeyValue> */
-            if ((ret = decode_drmrel_keyvalue(data)) != WBXML_OK)
-                return ret;
-
-            return WBXML_OK;
+            /* Decode base64 value */ 
+            return decode_base64_value(data);
         }
         break;
 
 #endif /* WBXML_SUPPORT_DRMREL */    
-    
+
+#if defined( WBXML_SUPPORT_SYNCML )
+
+    case WBXML_LANG_SYNCML_SYNCML10: 
+    case WBXML_LANG_SYNCML_SYNCML11: 
+    case WBXML_LANG_SYNCML_SYNCML12: 
+        /* NextNonce */
+        if ((parser->current_tag->wbxmlCodePage == 0x01) &&
+            (parser->current_tag->wbxmlToken == 0x10)) 
+        {
+            /* Decode base64 value */ 
+            return decode_base64_value(data);
+        }
+        break;
+
+#endif /* WBXML_SUPPORT_SYNCML */    
+
+    default:
+        /* NOP */
+        break;
+    } /* switch */
+  
+    return WBXML_OK;
+}
+
+
+/**
+ * @brief Decode an Opaque Attribute Value buffer
+ * @param parser The WBXML Parser
+ * @param data The Opaque data buffer
+ * @return WBXML_OK if OK, another error code otherwise
+ */
+static WBXMLError decode_opaque_attr_value(WBXMLParser *parser, WBXMLBuffer **data)
+{
+    switch (parser->langTable->langID) 
+    {
+#if defined( WBXML_SUPPORT_OTA_SETTINGS )
+
+    case WBXML_LANG_OTA_SETTINGS:
+    {
+        WBXMLError ret = WBXML_OK;
+        
+        /* Decode base64 value */
+        if ((ret = decode_base64_value(data)) != WBXML_OK)
+            return ret; 
+
+        return WBXML_OK;
+    }
+
+#endif /* WBXML_SUPPORT_OTA_SETTINGS */ 
+
     default:
         /* NOP */
         break;
@@ -2658,39 +2848,54 @@ static WBXMLError decode_wv_datetime(WBXMLBuffer **data)
     data_ptr = wbxml_buffer_get_cstr(*data);
   
     /* Get Year */
-    the_value = (WB_ULONG) ((data_ptr[0] << 6) | ((data_ptr[1] >> 2) && 0x3F));
+    the_value = (WB_ULONG) (((data_ptr[0] & 0x3F) << 6) + ((data_ptr[1] >> 2) & 0x3F));
     sprintf(the_year, "%u", the_value);
   
     /* Get Month */
-    the_value = (WB_ULONG) (((data_ptr[1] && 0x03) << 2) | ((data_ptr[2] >> 6) && 0x3F));
-    sprintf(the_month, "%u", the_value);
+    the_value = (WB_ULONG) (((data_ptr[1] & 0x03) << 2) | ((data_ptr[2] >> 6) & 0x03));
+    sprintf(the_month, "%02u", the_value);
   
     /* Get Day */
-    the_value = (WB_ULONG) ((data_ptr[2] >> 1) && 0x1F);
-    sprintf(the_date, "%u", the_value);
+    the_value = (WB_ULONG) ((data_ptr[2] >> 1) & 0x1F);
+    sprintf(the_date, "%02u", the_value);
   
     /* Get Hour */
-    the_value = (WB_ULONG) (((data_ptr[2] && 0x01) << 4) | ((data_ptr[3] >> 4) && 0x0F));
-    sprintf(the_hour, "%u", the_value);
+    the_value = (WB_ULONG) (((data_ptr[2] & 0x01) << 4) | ((data_ptr[3] >> 4) & 0x0F));
+    sprintf(the_hour, "%02u", the_value);
   
     /* Get Minute */
-    the_value = (WB_ULONG) (((data_ptr[3] && 0x0F) << 2) | ((data_ptr[4] >> 6) && 0x03));
-    sprintf(the_minute, "%u", the_value);
+    the_value = (WB_ULONG) (((data_ptr[3] & 0x0F) << 2) | ((data_ptr[4] >> 6) & 0x03));
+    sprintf(the_minute, "%02u", the_value);
   
     /* Get Second */
-    the_value = (WB_ULONG) (data_ptr[4] && 0x3F);
-    sprintf(the_second, "%u", the_value);
+    the_value = (WB_ULONG) (data_ptr[4] & 0x3F);
+    sprintf(the_second, "%02u", the_value);
   
     /* Get Time Zone */
-    if (data_ptr[5] == 'Z') {
+    if (data_ptr[5] == 0) {
+        /* This is a bug in the WBXML document.
+         * If timezone UTC aka Zulu is used then a 'Z' must be set.
+         */
         sprintf((WB_TINY *) result,
                 "%s%s%sT%s%s%sZ",
-                the_year, the_month, the_date, the_hour, the_minute, the_second);
+                the_year, the_month, the_date, the_hour, the_minute, the_value ? the_second : "");
+    } else if (data_ptr[5] < 'A' ||
+               data_ptr[5] > 'Z' ||
+               data_ptr[5] == 'J')
+    {
+	/* This is a bug in the WBXML document.
+	 * The timezone byte is set and wrong.
+         * There is no way to recover cleanly from this.
+         * Therefore no timezone is set.
+         */
+        sprintf((WB_TINY *) result,
+                "%s%s%sT%s%s%s",
+                the_year, the_month, the_date, the_hour, the_minute, the_value ? the_second : "");
     }
     else {
         sprintf((WB_TINY *) result,
-                "%s%s%sT%s%s%s",
-                the_year, the_month, the_date, the_hour, the_minute, the_second);
+                "%s%s%sT%s%s%s%c",
+                the_year, the_month, the_date, the_hour, the_minute, the_value ? the_second : "", data_ptr[5]);
     }
   
     /* Reset buffer */
@@ -2705,49 +2910,3 @@ static WBXMLError decode_wv_datetime(WBXMLBuffer **data)
 }
 
 #endif /* WBXML_SUPPORT_WV */
-
-
-#if defined( WBXML_SUPPORT_DRMREL )
-
-/**************************************
- * DRMREL 1.0
- */
-
-/**
- * @brief Decode a DRMREL 'KeyValue'
- * @param data The KeyValue to decode
- * @return WBXML_OK if OK, another error code otherwise
- * @note Used for:
- *      - DRMREL 1.0
- * @note [OMA-Download-DRMREL-v1_0-20020913-C.PDF] - 7.1:
- *       "The content of the <KeyValue> element MUST be in binary format using the opaque token."
- */
-static WBXMLError decode_drmrel_keyvalue(WBXMLBuffer **data)
-{
-    WB_UTINY   *result = NULL;
-    WBXMLError  ret    = WBXML_OK;
-  
-    if ((data == NULL) || (*data == NULL)) {
-        return WBXML_ERROR_INTERNAL;
-    }
-  
-    if ((result = wbxml_base64_encode((const WB_UTINY *) wbxml_buffer_get_cstr(*data),
-                                      wbxml_buffer_len(*data))) == NULL)
-    {
-        return WBXML_ERROR_B64_ENC;
-    }
-
-    /* Reset buffer */
-    wbxml_buffer_delete(*data, 0, wbxml_buffer_len(*data));
-
-    /* Set data */
-    if (!wbxml_buffer_append_cstr(*data, result)) {
-        ret = WBXML_ERROR_NOT_ENOUGH_MEMORY;
-    }
-
-    wbxml_free(result);
-  
-    return ret;
-}
-
-#endif /* WBXML_SUPPORT_DRMREL */
