@@ -1,6 +1,7 @@
 /*
  * libwbxml, the WBXML Library.
- * Copyright (C) 2002-2005 Aymerick Jehanne <aymerick@jehanne.org>
+ * Copyright (C) 2002-2008 Aymerick Jehanne <aymerick@jehanne.org>
+ * Copyright (C) 2011 Michael Bell <michael.bell@opensync.org>
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,7 +19,7 @@
  * 
  * LGPL v2.1: http://www.gnu.org/copyleft/lesser.txt
  * 
- * Contact: libwbxml@aymerick.com
+ * Contact: aymerick@jehanne.org
  * Home: http://libwbxml.aymerick.com
  */
  
@@ -26,13 +27,14 @@
  * @file wbxml_tree_clb_wbxml.c
  * @ingroup wbxml_tree
  *
- * @author Aymerick Jehanne <libwbxml@aymerick.com>
+ * @author Aymerick Jehanne <aymerick@jehanne.org>
  * @date 03/02/22
  *
  * @brief WBXML Tree Callbacks for WBXML Parser
  */
 
-#include "wbxml.h"
+#include "wbxml_tree_clb_wbxml.h"
+#include "wbxml_tree.h"
 
 
 /***************************************************
@@ -60,7 +62,7 @@ void wbxml_tree_clb_wbxml_end_document(void *ctx)
 }
 
 
-void wbxml_tree_clb_wbxml_start_element(void *ctx, WBXMLTag *element, WBXMLAttribute **attrs, WB_BOOL empty)
+void wbxml_tree_clb_wbxml_start_element(void *ctx, WBXMLTag *element, WBXMLAttribute **attrs)
 {
     WBXMLTreeClbCtx *tree_ctx = (WBXMLTreeClbCtx *) ctx;
 
@@ -78,7 +80,7 @@ void wbxml_tree_clb_wbxml_start_element(void *ctx, WBXMLTag *element, WBXMLAttri
 }
 
 
-void wbxml_tree_clb_wbxml_end_element(void *ctx, WBXMLTag *element, WB_BOOL empty)
+void wbxml_tree_clb_wbxml_end_element(void *ctx, WBXMLTag *element)
 {
     WBXMLTreeClbCtx *tree_ctx = (WBXMLTreeClbCtx *) ctx;
 
@@ -127,7 +129,7 @@ void wbxml_tree_clb_wbxml_characters(void *ctx, WB_UTINY *ch, WB_ULONG start, WB
     switch (wbxml_tree_node_get_syncml_data_type(tree_ctx->current)) {
     case WBXML_SYNCML_DATA_TYPE_WBXML:
         /* Deal with Embedded SyncML Documents - Parse WBXML */
-        if (wbxml_tree_from_wbxml(ch + start, length, WBXML_LANG_UNKNOWN, &tmp_tree) != WBXML_OK) {
+        if (wbxml_tree_from_wbxml(ch + start, length, WBXML_LANG_UNKNOWN, tree_ctx->tree->orig_charset, &tmp_tree) != WBXML_OK) {
             /* Not parsable ? Just add it as a Text Node... */
             goto text_node;
         }
