@@ -1,6 +1,7 @@
 /*
  * libwbxml, the WBXML Library.
- * Copyright (C) 2002-2005 Aymerick Jehanne <aymerick@jehanne.org>
+ * Copyright (C) 2002-2008 Aymerick Jehanne <aymerick@jehanne.org>
+ * Copyright (C) 2008-2011 Michael Bell <michael.bell@opensync.org>
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,7 +19,7 @@
  * 
  * LGPL v2.1: http://www.gnu.org/copyleft/lesser.txt
  * 
- * Contact: libwbxml@aymerick.com
+ * Contact: aymerick@jehanne.org
  * Home: http://libwbxml.aymerick.com
  */
  
@@ -26,16 +27,22 @@
  * @file wbxml_tree_clb_xml.c
  * @ingroup wbxml_tree
  *
- * @author Aymerick Jehanne <libwbxml@aymerick.com>
+ * @author Aymerick Jehanne <aymerick@jehanne.org>
  * @date 03/03/11
  *
  * @brief WBXML Tree Callbacks for XML Parser (Expat)
  */
 
+#include "wbxml_config_internals.h"
+
 #if defined( HAVE_EXPAT )
 
-#include "wbxml.h"
-
+#include "wbxml_tree_clb_xml.h"
+#include "wbxml_tree.h"
+#include "wbxml_log.h"
+#include "wbxml_charset.h"
+#include "wbxml_base64.h"
+#include <assert.h>
 
 /************************************
  *  Public Functions
@@ -48,6 +55,8 @@ void wbxml_tree_clb_xml_decl(void           *ctx,
 {
     WBXMLTreeClbCtx *tree_ctx = (WBXMLTreeClbCtx *) ctx;
 
+    (void) standalone; /* avoid warning about unused parameter */
+
     if (tree_ctx->expat_utf16) {
         /** @todo Convert from UTF-16 to UTF-8 */
     }
@@ -76,6 +85,9 @@ void wbxml_tree_clb_xml_doctype_decl(void           *ctx,
     WBXMLTreeClbCtx *tree_ctx = (WBXMLTreeClbCtx *) ctx;
     const WBXMLLangEntry *lang_table = NULL;
 
+    (void) doctypeName;         /* avoid warning about unused parameter */
+    (void) has_internal_subset; /* avoid warning about unused parameter */
+
     if (tree_ctx->expat_utf16) {
         /** @todo Convert from UTF-16 to UTF-8 */
     }
@@ -104,6 +116,8 @@ void wbxml_tree_clb_xml_start_element(void           *ctx,
     WBXMLTreeClbCtx *tree_ctx = (WBXMLTreeClbCtx *) ctx;
     const WBXMLLangEntry *lang_table = NULL;
 
+    WBXML_DEBUG((WBXML_PARSER, "Expat element start callback ('%s')", localName));
+
     if (tree_ctx->expat_utf16) {
         /** @todo Convert from UTF-16 to UTF-8 */
     }
@@ -141,8 +155,14 @@ void wbxml_tree_clb_xml_start_element(void           *ctx,
 
 #if defined( WBXML_SUPPORT_SYNCML )
 
-    /* If this is an embedded (not root) "DevInf" document, skip it */
-    if ((WBXML_STRCMP(localName, "DevInf") == 0) &&
+    /* If this is an embedded (not root) document, skip it
+     * Actually SyncML DevInf and DM DDF are known as such
+     * potentially embedded documents.
+     */
+    if ((
+         (WBXML_STRCMP(localName, "syncml:devinf:DevInf") == 0) ||
+         (WBXML_STRCMP(localName, "syncml:dmddf1.2:MgmtTree") == 0)
+        )&&
         (tree_ctx->current != NULL))
     {
         tree_ctx->skip_start = XML_GetCurrentByteIndex(tree_ctx->xml_parser);
@@ -171,11 +191,54 @@ void wbxml_tree_clb_xml_end_element(void           *ctx,
                                     const XML_Char *localName)
 {
     WBXMLTreeClbCtx *tree_ctx = (WBXMLTreeClbCtx *) ctx;
-#if defined( WBXML_SUPPORT_SYNCML )
-    WBXMLBuffer *devinf_doc = NULL;
-    WBXMLTree *tree = NULL;
+    WBXMLBuffer *content = NULL;
+    WBXMLTreeNode *node = NULL;
     WBXMLError ret = WBXML_OK;
-#endif /* WBXML_SUPPORT_SYNCML */
+
+    WBXML_DEBUG((WBXML_PARSER, "Expat element end callback ('%s')", localName));
+
+    /* If the node is flagged as binary node
+     * then the data is base64 encoded in the XML document
+     * and the data must be decoded in one step.
+     * Examples: Microsoft ActiveSync tags ConversationId or MIME
+     */
+
+    node = tree_ctx->current;
+    if (node && node->type == WBXML_TREE_ELEMENT_NODE &&
+        node->name->type == WBXML_VALUE_TOKEN &&
+        node->name->u.token->options & WBXML_TAG_OPTION_BINARY)
+    {
+        if (node->content == NULL)
+        {
+            WBXML_DEBUG((WBXML_PARSER, "    Binary tag: No content => no conversion!"));
+        } else {
+            WBXML_DEBUG((WBXML_PARSER, "    Binary tag: Convert base64 data"));
+            ret = wbxml_buffer_decode_base64(node->content);
+            if (ret != WBXML_OK)
+            {
+                WBXML_DEBUG((WBXML_PARSER, "    Binary tag: Base64 decoder failed!"));
+                tree_ctx->error = ret;
+            } else {
+                /* Add the buffer as a regular string node (since libwbxml doesn't
+                 * offer a way to specify an opaque data node). The WBXML
+                 * encoder is responsible for generating correct opaque data for
+                 * nodes like this.
+                 */
+                if (wbxml_tree_add_text(tree_ctx->tree,
+                                        tree_ctx->current,
+                                        (const WB_UTINY*)wbxml_buffer_get_cstr(node->content),
+                                        wbxml_buffer_len(node->content)) == NULL)
+                {
+                    WBXML_DEBUG((WBXML_PARSER, "    Binary tag: Cannot add base64 decoded node!"));
+                    tree_ctx->error = WBXML_ERROR_INTERNAL;
+                }
+            }
+            /* safe cleanup */
+            content = node->content;
+            node->content = NULL;
+            wbxml_buffer_destroy(content);
+        }
+    }
 
     if (tree_ctx->expat_utf16) {
         /** @todo Convert from UTF-16 to UTF-8 */
@@ -192,33 +255,98 @@ void wbxml_tree_clb_xml_end_element(void           *ctx,
             /* End of skipped node */
 
 #if defined( WBXML_SUPPORT_SYNCML )
-            if (WBXML_STRCMP(localName, "DevInf") == 0) {
-                /* Get embedded DevInf Document */
-                devinf_doc = wbxml_buffer_create(tree_ctx->input_buff + tree_ctx->skip_start, 
+            if (WBXML_STRCMP(localName, "syncml:devinf:DevInf") == 0 ||
+	        WBXML_STRCMP(localName, "syncml:dmddf1.2:MgmtTree") == 0) {
+		/* definitions first ... or some compilers don't like it */
+                WBXMLBuffer *embed_doc = NULL;
+                WBXMLTree *tree = NULL;
+		const WBXMLLangEntry *lang;
+
+                /* Get embedded DevInf or DM DDF Document */
+                embed_doc = wbxml_buffer_create(tree_ctx->input_buff + tree_ctx->skip_start, 
                                                  XML_GetCurrentByteIndex(tree_ctx->xml_parser) - tree_ctx->skip_start,
                                                  XML_GetCurrentByteIndex(tree_ctx->xml_parser) - tree_ctx->skip_start + 10);
+                if (embed_doc == NULL) {
+                    tree_ctx->error = WBXML_ERROR_NOT_ENOUGH_MEMORY;
+                    wbxml_buffer_destroy(embed_doc);
+                    return;
+                }
 
                 if (tree_ctx->expat_utf16) {
                     /** @todo Convert from UTF-16 to UTF-8 */
                 }
 
-                /* Check Buffer Creation and addd </DevInf> ending tag */
-                if ((devinf_doc == NULL) || (!wbxml_buffer_append_cstr(devinf_doc, "</DevInf>")))
+                /* Check Buffer Creation and add the closing tag */
+		if ((WBXML_STRCMP(localName, "syncml:devinf:DevInf") == 0 &&
+		     (!wbxml_buffer_append_cstr(embed_doc, "</DevInf>")))
+                    ||
+		    (WBXML_STRCMP(localName, "syncml:dmddf1.2:MgmtTree") == 0 &&
+		     (!wbxml_buffer_append_cstr(embed_doc, "</MgmtTree>"))))
                 {
                     tree_ctx->error = WBXML_ERROR_NOT_ENOUGH_MEMORY;
-                    wbxml_buffer_destroy(devinf_doc);
+                    wbxml_buffer_destroy(embed_doc);
                     return;
                 }
 
-                WBXML_DEBUG((WBXML_PARSER, "\t DevInf Doc : '%s'", wbxml_buffer_get_cstr(devinf_doc)));
+                /* Add doctype to give the XML parser a chance */
+		if (WBXML_STRCMP(localName, "syncml:dmddf1.2:MgmtTree") == 0 &&
+		    tree_ctx->tree->lang->langID != WBXML_LANG_SYNCML_SYNCML12)
+		{
+                    tree_ctx->error = WBXML_ERROR_UNKNOWN_XML_LANGUAGE;
+                    wbxml_buffer_destroy(embed_doc);
+                    return;
+		}
+		switch(tree_ctx->tree->lang->langID)
+		{
+			case WBXML_LANG_SYNCML_SYNCML10:
+				lang = wbxml_tables_get_table(WBXML_LANG_SYNCML_DEVINF10);
+				break;
+			case WBXML_LANG_SYNCML_SYNCML11:
+				lang = wbxml_tables_get_table(WBXML_LANG_SYNCML_DEVINF11);
+				break;
+			case WBXML_LANG_SYNCML_SYNCML12:
+				if (WBXML_STRCMP(localName, "syncml:dmddf1.2:MgmtTree") == 0) {
+					lang = wbxml_tables_get_table(WBXML_LANG_SYNCML_DMDDF12);
+				} else {
+					lang = wbxml_tables_get_table(WBXML_LANG_SYNCML_DEVINF12);
+				}
+				break;
+			default:
+				tree_ctx->error = WBXML_ERROR_UNKNOWN_XML_LANGUAGE;
+				wbxml_buffer_destroy(embed_doc);
+				return;
+		}
+
+		assert (lang!= NULL);
+		if (lang == NULL) {
+			tree_ctx->error = WBXML_ERROR_UNKNOWN_XML_LANGUAGE;
+			wbxml_buffer_destroy(embed_doc);
+			return;
+		}
+
+		/* DOCTYPE in reverse order */
+		if (!wbxml_buffer_insert_cstr(embed_doc,(WB_UTINY *) "\">\n", 0) ||                     /* > */
+		    !wbxml_buffer_insert_cstr(embed_doc, (WB_UTINY *) lang->publicID->xmlDTD, 0) ||      /* DTD */
+		    !wbxml_buffer_insert_cstr(embed_doc, (WB_UTINY *) "\" \"", 0) ||                     /* DTD */
+		    !wbxml_buffer_insert_cstr(embed_doc, (WB_UTINY *) lang->publicID->xmlPublicID, 0) || /* Public ID */
+		    !wbxml_buffer_insert_cstr(embed_doc, (WB_UTINY *) " PUBLIC \"", 0) ||                /*  PUBLIC " */
+		    !wbxml_buffer_insert_cstr(embed_doc, (WB_UTINY *) lang->publicID->xmlRootElt, 0) ||  /* Root Element */
+		    !wbxml_buffer_insert_cstr(embed_doc, (WB_UTINY *) "<!DOCTYPE ", 0))                  /* <!DOCTYPE */
+		{
+			tree_ctx->error = WBXML_ERROR_ENCODER_APPEND_DATA;
+                	wbxml_buffer_destroy(embed_doc);
+			return;
+		}
+
+                WBXML_DEBUG((WBXML_PARSER, "\t Embedded Doc : '%s'", wbxml_buffer_get_cstr(embed_doc)));
 
                 /* Parse 'DevInf' Document */
-                if ((ret = wbxml_tree_from_xml(wbxml_buffer_get_cstr(devinf_doc),
-                                               wbxml_buffer_len(devinf_doc),
+                if ((ret = wbxml_tree_from_xml(wbxml_buffer_get_cstr(embed_doc),
+                                               wbxml_buffer_len(embed_doc),
                                                &tree)) != WBXML_OK)
                 {
                     tree_ctx->error = ret;
-                    wbxml_buffer_destroy(devinf_doc);
+                    wbxml_buffer_destroy(embed_doc);
                     return;
                 }
 
@@ -230,12 +358,12 @@ void wbxml_tree_clb_xml_end_element(void           *ctx,
                 {
                     tree_ctx->error = WBXML_ERROR_INTERNAL;
                     wbxml_tree_destroy(tree);
-                    wbxml_buffer_destroy(devinf_doc);
+                    wbxml_buffer_destroy(embed_doc);
                     return;
                 }
 
                 /* Clean-up */
-                wbxml_buffer_destroy(devinf_doc);
+                wbxml_buffer_destroy(embed_doc);
                 tree_ctx->skip_lvl = 0;
             }
 #endif /* WBXML_SUPPORT_SYNCML */
@@ -327,8 +455,11 @@ void wbxml_tree_clb_xml_characters(void           *ctx,
                                    const XML_Char *ch,
                                    int             len)
 {
+    WBXMLTreeNode *node;
     WBXMLTreeClbCtx *tree_ctx = (WBXMLTreeClbCtx *) ctx;
 
+    WBXML_DEBUG((WBXML_PARSER, "Expat text callback"));
+
     if (tree_ctx->expat_utf16) {
         /** @todo Convert from UTF-16 to UTF-8 */
     }
@@ -344,11 +475,31 @@ void wbxml_tree_clb_xml_characters(void           *ctx,
 #if defined ( WBXML_SUPPORT_SYNCML )
     /* Specific treatment for SyncML */
     switch (wbxml_tree_node_get_syncml_data_type(tree_ctx->current)) {
-    case WBXML_SYNCML_DATA_TYPE_CLEAR:
     case WBXML_SYNCML_DATA_TYPE_DIRECTORY_VCARD:
     case WBXML_SYNCML_DATA_TYPE_VCALENDAR:
     case WBXML_SYNCML_DATA_TYPE_VCARD:
     case WBXML_SYNCML_DATA_TYPE_VOBJECT:
+        /* SyncML has some real design bugs
+         * because the authors of the specification did not understand XML.
+         *
+         * There must be a hack to preserve the CRLFs of vFormat objects.
+         * The only chance to do this is the detection of the vFormat itself
+         * and the conversion of every LF to a CRLF.
+         *
+         * The line breaks are always in a single text node.
+         * So a CR is appended to get a CRLF at the end.
+         */
+
+        if (len == 1 && ch[0] == '\n') /* line break - LF */
+        {
+            ch = "\r\n";
+            len = 2;
+        }
+
+        /* Do not break here.
+         * The CDATA handling is required for vFormat objects too.
+         */
+    case WBXML_SYNCML_DATA_TYPE_CLEAR:
         /*
          * Add a missing CDATA section node
          *
@@ -411,7 +562,7 @@ void wbxml_tree_clb_xml_characters(void           *ctx,
          *     </Data>
          *  ...
          *
-         * In this example, the spaces beetwen "]]>" and "</Data>" mustn't be added
+         * In this example, the spaces beetwen "]]>" and "</Data>" must not be added
          * to a CDATA section. 
          */
         if ((tree_ctx->current != NULL) && 
@@ -436,6 +587,28 @@ void wbxml_tree_clb_xml_characters(void           *ctx,
     } /* switch */
 #endif /* WBXML_SUPPORT_SYNCML */
 
+    /* We expect that "byte array" or BLOB types are 
+     * encoded in Base 64 in the XML code, since they may contain binary data.
+     */
+
+    node = tree_ctx->current;
+    if (node && node->type == WBXML_TREE_ELEMENT_NODE &&
+        node->name->type == WBXML_VALUE_TOKEN &&
+        node->name->u.token->options & WBXML_TAG_OPTION_BINARY)
+    {
+        WBXML_DEBUG((WBXML_PARSER, "    Binary tag: Caching base64 encoded data for later conversion."));
+        if (node->content == NULL)
+        {
+            node->content = wbxml_buffer_create(ch, len, 1);
+            if (node->content == NULL)
+                tree_ctx->error = WBXML_ERROR_NOT_ENOUGH_MEMORY;
+        } else {
+            if (!wbxml_buffer_append_data(node->content, ch, len))
+                tree_ctx->error = WBXML_ERROR_NOT_ENOUGH_MEMORY;
+        }
+        return;
+    }
+
     /* Add Text Node */
     if (wbxml_tree_add_text(tree_ctx->tree,
                             tree_ctx->current,
