@@ -1,6 +1,7 @@
 /*
  * libwbxml, the WBXML Library.
- * Copyright (C) 2002-2005 Aymerick Jehanne <aymerick@jehanne.org>
+ * Copyright (C) 2002-2008 Aymerick Jehanne <aymerick@jehanne.org>
+ * Copyright (C) 2011 Michael Bell <michael.bell@opensync.org>
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,7 +19,7 @@
  * 
  * LGPL v2.1: http://www.gnu.org/copyleft/lesser.txt
  * 
- * Contact: libwbxml@aymerick.com
+ * Contact: aymerick@jehanne.org
  * Home: http://libwbxml.aymerick.com
  */
 
@@ -26,7 +27,7 @@
  * @file wbxml2xml_tool.c
  * @ingroup wbxml2xml_tool
  *
- * @author Aymerick Jhanne <libwbxml@aymerick.com>
+ * @author Aymerick Jehanne <aymerick@jehanne.org>
  * @date 03/02/22
  *
  * @brief WBXML to XML Converter Tool
@@ -37,7 +38,18 @@
 #include <memory.h>
 #endif /* ! __SYMBIAN32__ */
 
-#include "wbxml.h" /* libwbxml2 */
+/* The real libwbxml include is:
+ *
+ * #include <wbxml/wbxml.h>
+ *
+ * We must use a direct reference to enforce the correct include.
+ */
+#include "../src/wbxml.h"
+
+#ifdef WBXML_USE_LEAKTRACKER
+#include "src/wbxml_mem.h"
+#endif
+
 #include "getopt.h"
 
 /*
@@ -50,7 +62,6 @@
 
 #define INPUT_BUFFER_SIZE 1000
 
-
 static WBXMLLanguage get_lang(const WB_TINY *lang)
 {
 #if defined( WBXML_SUPPORT_WML )
@@ -127,6 +138,8 @@ static WBXMLLanguage get_lang(const WB_TINY *lang)
         return WBXML_LANG_SYNCML_DEVINF12;
     if (WBXML_STRCMP(lang, "METINF12") == 0)
         return WBXML_LANG_SYNCML_METINF12;
+    if (WBXML_STRCMP(lang, "DMDDF12") == 0)
+        return WBXML_LANG_SYNCML_DMDDF12;
 #endif /* WBXML_SUPPORT_SYNCML */
 
 #if defined( WBXML_SUPPORT_WV )
@@ -139,17 +152,76 @@ static WBXMLLanguage get_lang(const WB_TINY *lang)
 #if defined( WBXML_SUPPORT_AIRSYNC )
     if (WBXML_STRCMP(lang, "AIRSYNC") == 0)
         return WBXML_LANG_AIRSYNC;
+    if (WBXML_STRCMP(lang, "ACTIVESYNC") == 0)
+        return WBXML_LANG_ACTIVESYNC;
 #endif /* WBXML_SUPPORT_AIRSYNC */
 
+#if defined( WBXML_SUPPORT_CONML )
+    if (WBXML_STRCMP(lang, "CONML") == 0)
+        return WBXML_LANG_CONML;
+#endif /* WBXML_SUPPORT_CONML */
+
     return WBXML_LANG_UNKNOWN;
 }
 
 
+static WBXMLCharsetMIBEnum get_charset(const WB_TINY *charset)
+{
+    /* The good old ASCII */
+
+    if (WBXML_STRCMP(charset, "ASCII") == 0)
+        return WBXML_CHARSET_US_ASCII;
+
+    /* ISO-8859 character sets */
+
+    if (WBXML_STRCMP(charset, "ISO-8859-1") == 0)
+        return WBXML_CHARSET_ISO_8859_1;
+    if (WBXML_STRCMP(charset, "ISO-8859-2") == 0)
+        return WBXML_CHARSET_ISO_8859_2;
+    if (WBXML_STRCMP(charset, "ISO-8859-3") == 0)
+        return WBXML_CHARSET_ISO_8859_3;
+    if (WBXML_STRCMP(charset, "ISO-8859-4") == 0)
+        return WBXML_CHARSET_ISO_8859_4;
+    if (WBXML_STRCMP(charset, "ISO-8859-5") == 0)
+        return WBXML_CHARSET_ISO_8859_5;
+    if (WBXML_STRCMP(charset, "ISO-8859-6") == 0)
+        return WBXML_CHARSET_ISO_8859_6;
+    if (WBXML_STRCMP(charset, "ISO-8859-7") == 0)
+        return WBXML_CHARSET_ISO_8859_7;
+    if (WBXML_STRCMP(charset, "ISO-8859-8") == 0)
+        return WBXML_CHARSET_ISO_8859_8;
+    if (WBXML_STRCMP(charset, "ISO-8859-9") == 0)
+        return WBXML_CHARSET_ISO_8859_9;
+    if (WBXML_STRCMP(charset, "ISO-10646-UCS-2") == 0)
+        return WBXML_CHARSET_ISO_10646_UCS_2;
+
+    /* Chinese character set */
+
+    if (WBXML_STRCMP(charset, "SHIFT_JIS") == 0)
+        return WBXML_CHARSET_SHIFT_JIS;
+
+    /* Japanese character set */
+
+    if (WBXML_STRCMP(charset, "BIG5") == 0)
+        return WBXML_CHARSET_BIG5;
+
+    /* Unicode character sets */
+
+    if (WBXML_STRCMP(charset, "UTF-8") == 0)
+        return WBXML_CHARSET_UTF_8;
+    if (WBXML_STRCMP(charset, "UTF-16") == 0)
+        return WBXML_CHARSET_UTF_16;
+
+    return WBXML_CHARSET_UNKNOWN;
+}
+
+
 static void help(void)
 {
-    fprintf(stderr, "wbxml2xml [libwbxml %s] by Aymerick Jehanne <aymerick@jehanne.org>\n", WBXML_LIB_VERSION);
-    fprintf(stderr, "If you use this tool, please send feedbacks to libwbxml@aymerick.com\n");
-    fprintf(stderr, "http://libwbxml.aymerick.com/\n");
+    fprintf(stderr, "wbxml2xml [libwbxml %s] by OpenSync\n", WBXML_LIB_VERSION);
+    fprintf(stderr, "This library was originally written by Aymerick Jehanne <aymerick@jehanne.org>\n");
+    fprintf(stderr, "If you use this tool, please send feedbacks to opensync-users@opensync.org\n");
+    fprintf(stderr, "http://libwbxml.opensync.org/\n");
 #if defined( HAVE_EXPAT )
     fprintf(stderr, "This tool is linked with Expat (http://expat.sourceforge.net)\n\n");
 #endif /* HAVE_EXPAT */
@@ -207,14 +279,35 @@ static void help(void)
     fprintf(stderr, "       SYNCML12 : SYNCML 1.2\n");
     fprintf(stderr, "       DEVINF12 : DEVINF 1.2\n");
     fprintf(stderr, "       METINF12 : METINF 1.2\n");
+    fprintf(stderr, "       DMDDF12  : DMDDF  1.2\n");
 #endif /* WBXML_SUPPORT_SYNCML */
 #if defined( WBXML_SUPPORT_WV )
     fprintf(stderr, "       CSP11 : WV CSP 1.1\n");
     fprintf(stderr, "       CSP12 : WV CSP 1.2\n");
 #endif /* WBXML_SUPPORT_WV */
 #if defined( WBXML_SUPPORT_AIRSYNC )
-    fprintf(stderr, "       AIRSYNC : Microsoft AirSync\n");
+    fprintf(stderr, "       AIRSYNC    : Microsoft AirSync    (SynCE namespaces) \n");
+    fprintf(stderr, "       ACTIVESYNC : Microsoft ActiveSync (original namespaces)\n");
 #endif /* WBXML_SUPPORT_AIRSYNC */
+#if defined( WBXML_SUPPORT_CONML )
+    fprintf(stderr, "       CONML : Nokia ConML\n");
+#endif /* WBXML_SUPPORT_CONML */
+    fprintf(stderr, "    -c X (Set character set if the document does not specify one)\n");
+    fprintf(stderr, "       ASCII           : US ASCII\n");
+    fprintf(stderr, "       ISO-8859-1      : ISO-8859-1 (Western European)\n");
+    fprintf(stderr, "       ISO-8859-2      : ISO-8859-2 (Central European)\n");
+    fprintf(stderr, "       ISO-8859-3      : ISO-8859-3 (South European)\n");
+    fprintf(stderr, "       ISO-8859-4      : ISO-8859-4 (North European)\n");
+    fprintf(stderr, "       ISO-8859-5      : ISO-8859-5 (Latin/Cyrillic)\n");
+    fprintf(stderr, "       ISO-8859-6      : ISO-8859-6 (Latin/Arabic)\n");
+    fprintf(stderr, "       ISO-8859-7      : ISO-8859-7 (Latin/Greek)\n");
+    fprintf(stderr, "       ISO-8859-8      : ISO-8859-8 (Latin/Hebrew)\n");
+    fprintf(stderr, "       ISO-8859-9      : ISO-8859-9 (Latin/Turkish)\n");
+    fprintf(stderr, "       ISO-10646-UCS-2 : UCS-2\n");
+    fprintf(stderr, "       SHIFT_JIS       : Shift JIS (Japanese character set)\n");
+    fprintf(stderr, "       BIG5            : Big5 (Chinese character set)\n");
+    fprintf(stderr, "       UTF-8           : UTF-8\n");
+    fprintf(stderr, "       UTF-16          : UTF-16\n");
     fprintf(stderr, "\nNote: '-' can be used to mean stdin on input or stdout on output\n\n");
 }
 
@@ -228,39 +321,43 @@ WB_LONG main(WB_LONG argc, WB_TINY **argv)
     WB_TINY opt;
     WBXMLError ret = WBXML_OK;
     WB_UTINY input_buffer[INPUT_BUFFER_SIZE + 1];
-    WBXMLGenXMLParams params;
+    WBXMLConvWBXML2XML *conv = NULL;
 
-    /* Init Default Parameters */
-    params.lang = WBXML_LANG_UNKNOWN;
-    params.gen_type = WBXML_GEN_XML_INDENT;
-    params.indent = 1;
-    params.keep_ignorable_ws = FALSE;
+    ret = wbxml_conv_wbxml2xml_create(&conv);
+    if (ret != WBXML_OK)
+    {
+        fprintf(stderr, "wbxml2xml failed: %s\n", wbxml_errors_string(ret));
+        goto clean_up;
+    }
 
-    while ((opt = (WB_TINY) getopt(argc, argv, "kh?o:m:i:l:")) != EOF)
+    while ((opt = (WB_TINY) wbxml_getopt(argc, argv, "kh?o:m:i:l:c:")) != EOF)
     {
         switch (opt) {
         case 'k':
-            params.keep_ignorable_ws = TRUE;
+            wbxml_conv_wbxml2xml_enable_preserve_whitespaces(conv);
             break;
         case 'i':
-            params.indent = (WB_UTINY) atoi((const WB_TINY*)optarg);
+            wbxml_conv_wbxml2xml_set_indent(conv, (WB_TINY) atoi((const WB_TINY*)optarg));
             break;
         case 'l':
-            params.lang = get_lang((const WB_TINY*)optarg);
+            wbxml_conv_wbxml2xml_set_language(conv, get_lang((const WB_TINY*)optarg));
+            break;
+        case 'c':
+            wbxml_conv_wbxml2xml_set_charset(conv, get_charset((const WB_TINY*)optarg));
             break;
         case 'm':
             switch (atoi((const WB_TINY*)optarg)) {
             case 0:
-                params.gen_type = WBXML_GEN_XML_COMPACT;
+                wbxml_conv_wbxml2xml_set_gen_type(conv, WBXML_GEN_XML_COMPACT);
                 break;
             case 1:
-                params.gen_type = WBXML_GEN_XML_INDENT;
+                wbxml_conv_wbxml2xml_set_gen_type(conv, WBXML_GEN_XML_INDENT);
                 break;
             case 2:
-                params.gen_type = WBXML_GEN_XML_CANONICAL;
+                wbxml_conv_wbxml2xml_set_gen_type(conv, WBXML_GEN_XML_CANONICAL);
                 break;
             default:
-                params.gen_type = WBXML_GEN_XML_INDENT;
+                wbxml_conv_wbxml2xml_set_gen_type(conv, WBXML_GEN_XML_INDENT);
             }
             break;
         case 'o':
@@ -270,14 +367,14 @@ WB_LONG main(WB_LONG argc, WB_TINY **argv)
         case '?':
         default:
             help();
-            return 0;
+            goto clean_up;
         }
     }
 
     if (optind >= argc) {
         fprintf(stderr, "Missing arguments\n");
         help();
-        return 0;
+        goto clean_up;
     }
 
 #ifdef WBXML_USE_LEAKTRACKER
@@ -309,18 +406,30 @@ WB_LONG main(WB_LONG argc, WB_TINY **argv)
             if (input_file != stdin)
                 fclose(input_file);
             if (wbxml != NULL)
+#ifdef WBXML_USE_LEAKTRACKER
                 wbxml_free(wbxml);
+#else
+                free(wbxml);
+#endif
             goto clean_up;
         }
 
         total += count;
+#ifdef WBXML_USE_LEAKTRACKER
         wbxml = wbxml_realloc(wbxml, total);
+#else
+        wbxml = realloc(wbxml, total);
+#endif
         if (wbxml == NULL) {
             fprintf(stderr, "Not enought memory\n");
             if (input_file != stdin)
                 fclose(input_file);
             if (wbxml != NULL)
+#ifdef WBXML_USE_LEAKTRACKER
                 wbxml_free(wbxml);
+#else
+                free(wbxml);
+#endif
             goto clean_up;
         }
 
@@ -332,7 +441,7 @@ WB_LONG main(WB_LONG argc, WB_TINY **argv)
         fclose(input_file);
 
     /* Convert WBXML document */
-    ret = wbxml_conv_wbxml2xml_withlen(wbxml, wbxml_len, &xml, &xml_len, &params);
+    ret = wbxml_conv_wbxml2xml_run(conv, wbxml, wbxml_len, &xml, &xml_len);
     if (ret != WBXML_OK) {
         fprintf(stderr, "wbxml2xml failed: %s\n", wbxml_errors_string(ret));
     }
@@ -365,18 +474,29 @@ WB_LONG main(WB_LONG argc, WB_TINY **argv)
         }
 
         /* Clean-up */
+#ifdef WBXML_USE_LEAKTRACKER
         wbxml_free(xml);
+#else
+        free(xml);
+#endif
     }
 
+#ifdef WBXML_USE_LEAKTRACKER
     wbxml_free(wbxml);
+#else
+    free(wbxml);
+#endif
 
 clean_up:
 
+    if (conv)
+        wbxml_conv_wbxml2xml_destroy(conv);
+
 #ifdef WBXML_USE_LEAKTRACKER
     lt_check_leaks();
     lt_shutdown_mem();
     lt_log_close_file();
 #endif
 
-    return 0;
+    return ret;
 }
