@@ -1,24 +1,26 @@
 /*
  * libwbxml, the WBXML Library.
- * Copyright (C) 2002-2005 Aymerick Jhanne <aymerick@jehanne.org>
+ * Copyright (C) 2002-2008 Aymerick Jehanne <aymerick@jehanne.org>
+ * Copyright (C) 2011 Michael Bell <michael.bell@opensync.org>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ * 
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ * 
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * 
+ * LGPL v2.1: http://www.gnu.org/copyleft/lesser.txt
+ * 
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * GPL v2: http://www.gnu.org/licenses/gpl.txt
- *
- * Contact: libwbxml@aymerick.com
+ * Contact: aymerick@jehanne.org
  * Home: http://libwbxml.aymerick.com
  */
 
@@ -26,7 +28,7 @@
  * @file xml2wbxml_tool.c
  * @ingroup xml2wbxml_tool
  *
- * @author Aymerick Jhanne <libwbxml@aymerick.com>
+ * @author Aymerick Jehanne <aymerick@jehanne.org>
  * @date 03/03/11
  *
  * @brief XML to WBXML Converter Tool
@@ -37,7 +39,18 @@
 #include <memory.h>
 #endif /* ! __SYMBIAN32__ */
 
-#include "wbxml.h" /* libwbxml2 */
+/* The real libwbxml include is:
+ *
+ * #include <wbxml/wbxml.h>
+ *
+ * We must use a direct reference to enforce the correct include.
+ */
+#include "../src/wbxml.h"
+
+#ifdef WBXML_USE_LEAKTRACKER
+#include "src/wbxml_mem.h"
+#endif
+
 #include "getopt.h"
 
 /*
@@ -67,9 +80,10 @@ static WBXMLVersion get_version(const WB_TINY *lang)
 
 
 static void help(void) {
-    fprintf(stderr, "xml2wbxml [libwbxml %s] by Aymerick Jehanne <aymerick@jehanne.org>\n", WBXML_LIB_VERSION);
-    fprintf(stderr, "If you use this tool, please send feedbacks to libwbxml@aymerick.com\n");
-    fprintf(stderr, "http://libwbxml.aymerick.com/\n");
+    fprintf(stderr, "xml2wbxml [libwbxml %s] by OpenSync\n", WBXML_LIB_VERSION);
+    fprintf(stderr, "This library was originally written by Aymerick Jehanne <aymerick@jehanne.org>\n");
+    fprintf(stderr, "If you use this tool, please send feedbacks to opensync-users@opensync.org\n");
+    fprintf(stderr, "http://libwbxml.opensync.org/\n");
 #if defined( HAVE_EXPAT )
     fprintf(stderr, "This tool is linked with Expat (http://expat.sourceforge.net)\n\n");
 #endif /* HAVE_EXPAT */
@@ -85,6 +99,8 @@ static void help(void) {
     fprintf(stderr, "       1.1 : WBXML 1.1\n");
     fprintf(stderr, "       1.2 : WBXML 1.2\n");
     fprintf(stderr, "       1.3 : WBXML 1.3\n");
+    fprintf(stderr, "    -a : anonymous format\n");
+    fprintf(stderr, "         do NOT include any format information (e.g. public ID or DTD)\n");
     fprintf(stderr, "\nNote: '-' can be used to mean stdin on input or stdout on output\n\n");
 }
 
@@ -98,24 +114,30 @@ WB_LONG main(WB_LONG argc, WB_TINY **argv)
     WB_TINY opt;
     WBXMLError ret = WBXML_OK;
     WB_UTINY input_buffer[INPUT_BUFFER_SIZE + 1];
-    WBXMLGenWBXMLParams params;
+    WBXMLConvXML2WBXML *conv = NULL;
 
-    /* Init Default Parameters */
-    params.wbxml_version = WBXML_VERSION_13;
-    params.use_strtbl = TRUE;
-    params.keep_ignorable_ws = FALSE;
+    ret = wbxml_conv_xml2wbxml_create(&conv);
+    if (ret != WBXML_OK)
+    {
+        fprintf(stderr, "xml2wbxml failed: %s\n", wbxml_errors_string(ret));
+        goto clean_up;
+    }
 
-    while ((opt = (WB_TINY) getopt(argc, argv, "nkh?o:v:")) != EOF)
+
+    while ((opt = (WB_TINY) wbxml_getopt(argc, argv, "nkah?o:v:")) != EOF)
     {
         switch (opt) {
         case 'v':
-            params.wbxml_version = get_version((const WB_TINY*)optarg);
+            wbxml_conv_xml2wbxml_set_version(conv, get_version((const WB_TINY*)optarg));
             break;
         case 'n':
-            params.use_strtbl = FALSE;
+            wbxml_conv_xml2wbxml_disable_string_table(conv);
             break;
         case 'k':
-            params.keep_ignorable_ws = TRUE;
+            wbxml_conv_xml2wbxml_enable_preserve_whitespaces(conv);
+            break;
+        case 'a':
+            wbxml_conv_xml2wbxml_disable_public_id(conv);
             break;
         case 'o':
             output = (WB_UTINY*) optarg;
@@ -124,14 +146,14 @@ WB_LONG main(WB_LONG argc, WB_TINY **argv)
         case '?':
         default:
             help();
-            return 0;
+            goto clean_up;
         }
     }
 
     if (optind >= argc) {
         fprintf(stderr, "Missing arguments\n");
         help();
-        return 0;
+        goto clean_up;
     }
 
 #ifdef WBXML_USE_LEAKTRACKER
@@ -163,12 +185,20 @@ WB_LONG main(WB_LONG argc, WB_TINY **argv)
             if (input_file != stdin)
                 fclose(input_file);
             if (xml != NULL)
+#ifdef WBXML_USE_LEAKTRACKER
                 wbxml_free(xml);
+#else
+                free(xml);
+#endif
             goto clean_up;
         }
 
         total += count;
+#ifdef WBXML_USE_LEAKTRACKER
         xml = wbxml_realloc(xml, total + 1);
+#else
+        xml = realloc(xml, total + 1);
+#endif
         if (xml == NULL) {
             fprintf(stderr, "Not enought memory\n");
             if (input_file != stdin)
@@ -186,7 +216,7 @@ WB_LONG main(WB_LONG argc, WB_TINY **argv)
     xml[xml_len] = '\0';
 
     /* Convert XML document */
-    ret = wbxml_conv_xml2wbxml_withlen(xml, xml_len, &wbxml, &wbxml_len, &params);
+    ret = wbxml_conv_xml2wbxml_run(conv, xml, xml_len, &wbxml, &wbxml_len);
     if (ret != WBXML_OK) {
         fprintf(stderr, "xml2wbxml failed: %s\n", wbxml_errors_string(ret));
     }
@@ -221,19 +251,30 @@ WB_LONG main(WB_LONG argc, WB_TINY **argv)
 
         /* Clean-up */
         if (wbxml != NULL)
+#ifdef WBXML_USE_LEAKTRACKER
             wbxml_free(wbxml);
+#else
+            free(wbxml);
+#endif
     }
 
     if (xml != NULL)
+#ifdef WBXML_USE_LEAKTRACKER
         wbxml_free(xml);
+#else
+       free(xml);
+#endif
 
 clean_up:
 
+    if (conv != NULL)
+        wbxml_conv_xml2wbxml_destroy(conv);
+
 #ifdef WBXML_USE_LEAKTRACKER
     lt_check_leaks();
     lt_shutdown_mem();
     lt_log_close_file();
 #endif
 
-    return 0;
+    return ret;
 }
